/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package NOD;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Hashtable;

import org.biojava.bio.seq.Sequence;

/**
 * 
 * @author mscott
 */
public class BatchmanFormatter {

	// int patternCount;
	String dir;
	int patternLength;
	int increment;
	String formattedPredictions;
	String[] protSegments;
	boolean[] nols;
	double[] prediction_scores;

	public BatchmanFormatter() throws Exception {
		// patternCount = 0;
	}

	public void setParameters(String d, int patternL, int step)
			throws Exception {
		dir = d;
		patternLength = patternL;
		increment = step;
	}

	public int printPatternsIntoNegTestFile(String prot_seq, String name,
			int patternLength, String[] aa_groups, String outputPattern,
			int increment, int id, String[][] jnetPredictions) throws Exception {
		try {

			int patternCount = 0;
			int substringCount = 0;
			String currentResidue;
			String patternFile = new String();
			// int setNumber = 0;
			for (int i = 0; i <= prot_seq.length() - patternLength; i = i
					+ increment) {
				patternCount++;
				substringCount++;
				patternFile += "\n# Input pattern " + patternCount + ": ";
				patternFile += name;
				patternFile += "-substring " + substringCount + "\n";
				for (int k = 0; k < patternLength; k++) {
					currentResidue = prot_seq.substring(i + k, i + k + 1);
					currentResidue.toUpperCase();
					for (int j = 0; j < aa_groups.length; j++) {
						if (aa_groups[j].indexOf(currentResidue) != -1) {
							// then currentResidue belongs to this aa class
							patternFile += "1 ";
						} else {
							patternFile += "0 ";
						}
					}
				}

				// then add structural info here:
				patternFile += printStructureInfo(
						prot_seq.substring(i, i + patternLength), name,
						prot_seq, jnetPredictions);

				patternFile += "\n# Output pattern " + patternCount + ": ";
				patternFile += name + "\n";
				patternFile += outputPattern;

				if (i % 25 == 0) {
					PrintWriter out = new PrintWriter(new BufferedWriter(
							new FileWriter(dir + "TempPatternFile_" + id
									+ ".txt", true)));
					out.print(patternFile);
					out.close();
					patternFile = new String();
				}
			}

			PrintWriter out = new PrintWriter(
					new BufferedWriter(new FileWriter(dir + "TempPatternFile_"
							+ id + ".txt", true)));
			out.print(patternFile);
			out.close();
			patternFile = null;

			return patternCount;

		} catch (Exception e) {
			e.printStackTrace();
			return -1;
		}

	}

	public String printStructureInfo(String prot_subseq, String name,
			String fullLengthSeq, String[][] jnetPredictions) throws Exception {
		try {

			String secStruct = new String();
			String currentSubSeq, id;
			String ResidueAA, JnetPred, JnetConf, JnetSol25, JnetSol5, JnetSol0;
			Sequence seq;
			double middle, closestToEnd, lengthScore;
			int countH = 0;
			int countE = 0;
			int countDash = 0;
			int countJnetSol25B = 0;
			int countJnetSol5B = 0;
			int countJnetSol0B = 0;
			int confidence = 0;
			double length = prot_subseq.length();
			boolean hasJpred = false;
			int extension, s, e;
			// String[][] jnetPredictions;

			middle = fullLengthSeq.length() / 2.0;
			for (int i = 0; i < fullLengthSeq.length() - prot_subseq.length()
					+ 1; i++) {
				currentSubSeq = fullLengthSeq.substring(i,
						i + prot_subseq.length());
				if (currentSubSeq.equalsIgnoreCase(prot_subseq)) {
					// we found the start position in the full length seq, it is
					// i+1
					// System.out.println("found NoLS in full-length prot at " +
					// i);
					i++;

					extension = (int) java.lang.Math.floor((20 - prot_subseq
							.length()) / 2);
					if (extension < 0) {
						s = i;
						e = i + prot_subseq.length();
					} else if ((i - extension) < 1) { // then i is at the
														// beginning of the
														// protein
						s = 1;
						e = i + prot_subseq.length() + extension;
					} else if ((i + prot_subseq.length() + extension) > fullLengthSeq
							.length()) { // then i is at the end of the protein
						s = i - extension;
						e = fullLengthSeq.length();
					} else {
						s = i - extension;
						e = i + prot_subseq.length() + extension;
					}

					// to calculate a score d of the distance to the ends:
					if (middle - i > i + prot_subseq.length() - middle) {
						closestToEnd = middle - i;
					} else {
						closestToEnd = i + prot_subseq.length() - middle;
						closestToEnd--;
					}
					secStruct += closestToEnd / middle + " ";

					// to calculate a score of the length of the protein:
					if (fullLengthSeq.length() > 406) {
						lengthScore = 1.0;
					} else {
						lengthScore = 1.0 - ((406.0 - fullLengthSeq.length()) / 406.0);
					}
					secStruct += lengthScore + " ";

					// now, read jnet info from file:
					// jnetPredictions = parseJnetFile(jnetFile);
					if (jnetPredictions.length > 2) {

						for (int j = s; j < e; j++) {

							JnetPred = jnetPredictions[0][j - 1];
							JnetConf = jnetPredictions[1][j - 1];
							JnetSol25 = jnetPredictions[2][j - 1];
							JnetSol5 = jnetPredictions[3][j - 1];
							JnetSol0 = jnetPredictions[4][j - 1];

							if (JnetPred.equalsIgnoreCase("H")) {
								countH++;
							} else if (JnetPred.equalsIgnoreCase("E")) {
								countE++;
							} else if (JnetPred.equalsIgnoreCase("-")) {
								countDash++;
							}

							confidence += Integer.parseInt(JnetConf);

							if (JnetSol25.equalsIgnoreCase("B")) {
								countJnetSol25B++;
							}
							if (JnetSol5.equalsIgnoreCase("B")) {
								countJnetSol5B++;
							}
							if (JnetSol0.equalsIgnoreCase("B")) {
								countJnetSol0B++;
							}
						}

						length = e - s;
						secStruct += countH / length + " ";
						secStruct += countE / length + " ";
						secStruct += countDash / length + " ";
						secStruct += confidence / (length * 9) + " ";
						secStruct += countJnetSol25B / length + " ";
						secStruct += countJnetSol5B / length + " ";
						secStruct += countJnetSol0B / length + " ";

					} else {
						secStruct += "0 0 0 0 0 0 0";
					}

					i = fullLengthSeq.length();
				}

			}

			return secStruct;
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/*
	 * Modified from Mark's jnet parser
	 */
	public String[][] parseJnetFile(File jnetFile) throws Exception {
		try {

			FileReader r = new FileReader(jnetFile);
			BufferedReader rStream = new BufferedReader(r);

			String line = "";
			Hashtable jpred = new Hashtable(1, 1);
			String[] sline;
			String[][] jnetParsed;

			boolean jpredLoaded = false;

			while ((line = rStream.readLine()) != null) {
				// System.out.println(line.length() + "\t\"" + line + "\"");
				if (line.length() > 0) {
					sline = line.split(":");
					if (sline[0].matches("jnetpred")) {
						String[] jnetpred = sline[1].split(",");
						jpred.put(sline[0], jnetpred);
					} else if (sline[0].matches("JNETCONF")) {
						String[] jnetconf = sline[1].split(",");
						jpred.put(sline[0], jnetconf);
					} else if (sline[0].matches("JNETSOL25")) {
						String[] jnetsol25 = sline[1].split(",");
						jpred.put(sline[0], jnetsol25);
					} else if (sline[0].matches("JNETSOL5")) {
						String[] jnetsol5 = sline[1].split(",");
						jpred.put(sline[0], jnetsol5);
					} else if (sline[0].matches("JNETSOL0")) {
						String[] jnetsol0 = sline[1].split(",");
						jpred.put(sline[0], jnetsol0);
						jpredLoaded = true;
					}

					// don't care about the rest of the predictions:
					/*
					 * else if (sline[0].matches("JNETHMM")) { String[] jnethmm
					 * = sline[1].split(","); jpred.put(sline[0], jnethmm); }
					 * else if (sline[0].matches("JNETPSSM")) { String[]
					 * jnetpssm = sline[1].split(","); jpred.put(sline[0],
					 * jnetpssm); } else if (sline[0].matches("JNETJURY")) {
					 * String[] jnetjury = sline[1].split(",");
					 * jpred.put(sline[0], jnetjury); } else if
					 * (sline[0].matches("JNETPROPH")) { String[] jnetproph =
					 * sline[1].split(","); jpred.put(sline[0], jnetproph); }
					 * else if (sline[0].matches("JNETPROPE")) { String[]
					 * jnetprope = sline[1].split(","); jpred.put(sline[0],
					 * jnetprope); } else if (sline[0].matches("JNETPROPC")) {
					 * String[] jnetpropc = sline[1].split(",");
					 * jpred.put(sline[0], jnetpropc); jpredLoaded = true; }
					 */
				}
			}
			rStream.close();
			r.close();

			if (jpredLoaded) {

				String[] jnetpred = (String[]) jpred.get("jnetpred");
				String[] jnetconf = (String[]) jpred.get("JNETCONF");
				String[] jnetsol25 = (String[]) jpred.get("JNETSOL25");
				String[] jnetsol5 = (String[]) jpred.get("JNETSOL5");
				String[] jnetsol0 = (String[]) jpred.get("JNETSOL0");
				/*
				 * String[] jnethmm = (String[])jpred.get("JNETHMM"); String[]
				 * jnetpssm = (String[])jpred.get("JNETPSSM"); String[] jnetjury
				 * = (String[])jpred.get("JNETJURY"); String[] jnetproph =
				 * (String[])jpred.get("JNETPROPH"); String[] jnetprope =
				 * (String[])jpred.get("JNETPROPE"); String[] jnetpropc =
				 * (String[])jpred.get("JNETPROPC");
				 */

				jnetParsed = new String[5][jnetpred.length];
				jnetParsed[0] = jnetpred;
				jnetParsed[1] = jnetconf;
				jnetParsed[2] = jnetsol25;
				jnetParsed[3] = jnetsol5;
				jnetParsed[4] = jnetsol0;

				return jnetParsed;
			} else {
				return new String[1][1];
			}

		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public String formatPredictions(int id, String name, String prot_seq,
			int patternCount, File predFile) throws Exception {
		try {
			// String extension = "_net4000run1_predictions";
			boolean nameFound;
			boolean targetFctFound;
			int targetFunction;
			double score;
			Hashtable patternScores;
			double[] scores = new double[patternCount + 1]; // these are the
															// scores for each
															// of the substrings
			java.util.Arrays.fill(scores, -1.0);
			scores[0] = 0;
			int index, patternNum;
			name = null;
			double highest_average;
			double t, min, max;
			Enumeration enu;
			int[] positives;
			int[] negativesAll;
			int[] negativesNLS;
			int[] negativesNuc;
			int[] negativesCyto;
			BufferedReader in;
			String seqName, line;
			int residuePos;
			int consecutiveRes = 8;
			double[] nols_positions_scores = new double[prot_seq.length()];
			java.util.Arrays.fill(nols_positions_scores, 0.0);
			int[] starts;
			int count;
			formattedPredictions = new String();

			in = new BufferedReader(new FileReader(predFile));
			// seqName = null;
			nameFound = false;
			targetFctFound = false;
			targetFunction = -1;
			score = -1.0;
			patternNum = -1;
			// patternScores = new Hashtable();
			// scores = null;
			while ((line = in.readLine()) != null) {
				if (line.indexOf("#") != -1) {
					patternNum = Integer.parseInt(line.substring(1,
							line.indexOf(".")));
					// seqName = names[patternNum];
					nameFound = true;
				} else if (nameFound) { // the next line after the seq ID is the
										// target fct
					nameFound = false;
					targetFctFound = true;
					targetFunction = Integer.parseInt(line);
				} else if (targetFctFound) { // the next line after the target
												// fct is the neural network
												// output (the score)
					targetFctFound = false;
					score = Double.parseDouble(line);
					scores[patternNum] = score;

					/*
					 * for (index=0; index<scores.length; index++){ if
					 * (scores[index] < 0.0){ scores[index] = score; index =
					 * scores.length; }
					 */
				}
			}

			// now, we have read all the scores output by the neural network
			// let's fill in the NoLSResPredictions table
			// for (int p=1; p<scores.length; p++){ //p starts at 1 because
			// scores[0] is the target fct, not a score output by the NN
			// residuePos = (p-1) + 7; //the second term should correspond to
			// (patternLength + 1) / 2 to point to the middle position of the
			// window
			// /*nolsResPredictions += name + "\t" + residuePos + "\t" +
			// prot_seq.charAt(residuePos-1);
			// nolsResPredictions += "\t" + scores[p] + "\n";*/
			// formattedPrediction += residuePos + "\t" +
			// prot_seq.charAt(residuePos-1) + "\t" + scores[p] + "\n";
			// }
			// formattedPrediction +=
			// "\n\nNoLS predictions (start positions):\n";

			// now, we need to find all regions in this protein that score high
			// enough to be predict as an NoLS:
			// enu = patternScores.keys();
			positives = new int[20];
			negativesAll = new int[20];
			negativesNLS = new int[20];
			negativesNuc = new int[20];
			negativesCyto = new int[20];
			java.util.Arrays.fill(positives, 0);
			java.util.Arrays.fill(negativesAll, 0);
			java.util.Arrays.fill(negativesNLS, 0);
			java.util.Arrays.fill(negativesNuc, 0);
			java.util.Arrays.fill(negativesCyto, 0);
			// while (enu.hasMoreElements()){
			// seqName = (String)enu.nextElement();
			// scores = (double[])patternScores.get(seqName);
			highest_average = 0.0;

			// here, need to remember all high scoring regions, not just the
			// highest
			count = 0;
			for (int n = 1; n < scores.length - (consecutiveRes - 1); n++) {
				t = 0;
				for (int j = 0; j < consecutiveRes; j++) {
					t += scores[n + j];
				}
				highest_average = t / consecutiveRes;
				nols_positions_scores[n] = highest_average;
				if (highest_average > 0.8) {
					// then this region is predicted to be a NoLS
					// formattedPrediction += n + "\t" + highest_average + "\n";
					count++;
				}
			}

			prediction_scores = new double[nols_positions_scores.length - 19];
			for (int i = 0; i < nols_positions_scores.length - 19; i++) {
				prediction_scores[i] = nols_positions_scores[i];
			}

			// create an array of int representing the start positions of every
			// NoLSs in the protein to use code from formatPredictions() from
			// NolsFormatter:
			starts = new int[count];
			count = 0;
			for (int i = 0; i < nols_positions_scores.length; i++) {
				if (nols_positions_scores[i] > 0.8) {
					starts[count] = i - 1;
					count++;
				}
			}

			// code from formatPredictions() from NolsFormatter:
			int previous_end = 0;
			int start = starts[0];
			int count_segments = 0;
			if (start > 1) {
				// nols[0] = false;
				// protSegments[0] = AA_seq.substring(1, start);
				count_segments++;
			} else {
				// nols[0] = true;
			}
			int end = start + 20;
			count = 0;

			for (int i = 1; i < starts.length; i++) {
				if (starts[i] - starts[i - 1] < 21) {
					end = starts[i] + 20;
				} else {
					if (previous_end > 0) {
						count_segments++;
					}
					// need to print out start i-1 info
					formattedPredictions += "<tt>"
							+ prot_seq.substring(start, end)
							+ "</tt> (between positions " + (start + 1)
							+ " and " + end + ")<br><br>";
					count++;
					count_segments++;

					previous_end = end;
					start = starts[i];
					end = start + 20;
				}
			}

			if (previous_end > 0) {
				count_segments++;
			}
			count_segments++;
			if (end < prot_seq.length()) {
				count_segments++;
			}

			// System.out.println(end);

			/*
			 * formattedPredictions += "NoLS predicted between positions " +
			 * start + " and " + end + ": " + AA_seq.substring(start, end) +
			 * "<br><br>";
			 */
			formattedPredictions += "<tt>" + prot_seq.substring(start, end)
					+ "</tt> (between positions " + (start + 1) + " and " + end
					+ ")<br><br>";

			count++;

			if (count == 1) {
				formattedPredictions = "<strong>One NoLS is predicted in this protein:</strong><br><br>"
						+ formattedPredictions;
			} else {
				formattedPredictions = "<strong>"
						+ count
						+ " NoLSs are predicted in this protein:</strong><br><br>"
						+ formattedPredictions;
			}

			protSegments = new String[count_segments];
			nols = new boolean[count_segments];

			previous_end = 0;
			start = starts[0];
			count_segments = 0;
			if (start > 1) {
				nols[0] = false;
				protSegments[0] = prot_seq.substring(0, start);
				count_segments++;
			} else {
				nols[0] = true;
			}
			end = start + 20;
			for (int i = 1; i < starts.length; i++) {
				if (starts[i] - starts[i - 1] < 21) {
					end = starts[i] + 20;
				} else {
					// need to print out start i-1 info
					if (previous_end > 0) {
						protSegments[count_segments] = prot_seq.substring(
								previous_end, start);
						nols[count_segments] = false;
						count_segments++;
					}
					count++;
					nols[count_segments] = true;
					protSegments[count_segments] = prot_seq.substring(start,
							end);
					count_segments++;

					previous_end = end;
					start = starts[i];
					end = start + 20;
				}
			}

			if (previous_end > 0) {
				protSegments[count_segments] = prot_seq.substring(previous_end,
						start);
				nols[count_segments] = false;
				count_segments++;
			}
			// formattedPredictions += AA_seq.substring(start-1, end) +
			// " (between positions " + start + " and " + end + ")<br><br>";
			nols[count_segments] = true;
			protSegments[count_segments] = prot_seq.substring(start, end);
			count_segments++;
			if (end < prot_seq.length()) {
				nols[count_segments] = false;
				protSegments[count_segments] = prot_seq.substring(end);
				// count_segments++;
			}

			return formattedPredictions;

		}

		catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	public String[] getProtSegments() {
		if (protSegments.length > 0) {
			return protSegments;
		} else {
			return null;
		}
	}

	public boolean[] getNoLSs() {
		if (nols.length > 0) {
			return nols;
		} else {
			return new boolean[0];
		}
	}

	public double[] getScores() {
		return prediction_scores;
	}

}
