package NOD;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.geom.Ellipse2D;
import java.io.File;
import java.io.IOException;

import javax.swing.JPanel;

import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.IntervalMarker;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.DefaultXYDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.ui.Layer;
import org.jfree.ui.RectangleAnchor;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.TextAnchor;

/**
 * NOLS chart generator
 * 
 */
public class NolsChart {

	private static final Dimension imgSize = new Dimension(650, 400);
	private final String imgPath;

	/**
	 * 
	 * @param imageDirPath
	 *            the path to the directory where the generated images should be
	 *            stored
	 */

	/*
	 * public NolsChart(String imageDirPath) { if (imageDirPath == null) { throw
	 * new NullPointerException(
	 * "Directory for image file storage must be provided!"); } File dir = new
	 * File(imageDirPath); if (!dir.exists() || !new
	 * File(imageDirPath).isDirectory()) { throw new IllegalArgumentException(
	 * "The path provided does not exist or is not a directory!"); } imgPath =
	 * imageDirPath; }
	 */

	public NolsChart(String imageDirPath) {
		if (imageDirPath == null) {
			throw new NullPointerException(
					"Directory for image file storage must be provided!");
		}
		File dir = new File(imageDirPath);
		if (!dir.exists() || !new File(imageDirPath).isDirectory()) {
			throw new IllegalArgumentException(
					"The path provided does not exist or is not a directory! Value: "
							+ imageDirPath);
		}
		imgPath = imageDirPath;
	}

	/**
	 * Generates the chart and saves it in to the PNG file stored in the
	 * directory {@code imgPath}. The fully specified name of the generated file
	 * (together with path) is returned by the method.
	 * 
	 * @param data
	 *            NOLS predictions data, the position assume to correspond to
	 *            the index of the array
	 * @return the full path to the generated file
	 * @throws IOException
	 *             if the method cannot read/write into the file system
	 */
	public String createChart(double[] data) throws IOException {

		JPanel jpanel = new JPanel();
		jpanel.setPreferredSize(imgSize);

		// Axis
		NumberAxis xaxis = new NumberAxis("Position");
		xaxis.setAutoRangeIncludesZero(true);
		NumberAxis yaxis = new NumberAxis("Score");
		yaxis.setAutoRangeIncludesZero(false);
		yaxis.setRange(0, 1);
		// Range marker
		IntervalMarker intervalmarker = new IntervalMarker(0.8, 1d);
		// intervalmarker.setLabel("NoLS Candidate Signal");
		intervalmarker.setLabelFont(new Font("SansSerif", 2, 11));
		intervalmarker.setLabelAnchor(RectangleAnchor.LEFT);
		intervalmarker.setLabelTextAnchor(TextAnchor.CENTER_LEFT);
		intervalmarker.setPaint(new Color(255, 191, 191, 128)); // new
																// Color(222,
																// 222, 255,
																// 128) -
																// light blue
		// Data range renderer
		// XYSplineRenderer xysplinerenderer = new XYSplineRenderer();
		Ellipse2D.Double double1 = new Ellipse2D.Double(-3D, -3D, 6D, 6D);
		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setBaseLinesVisible(true);
		renderer.setBaseShapesVisible(true);
		renderer.setSeriesShape(0, double1);
		renderer.setSeriesPaint(0, Color.BLUE);
		renderer.setSeriesFillPaint(0, Color.WHITE);
		renderer.setSeriesOutlinePaint(0, Color.BLUE);
		renderer.setUseFillPaint(true);
		// Plot
		XYPlot xyplot = new XYPlot(convertData(data), xaxis, yaxis, renderer);
		xyplot.setBackgroundPaint(Color.WHITE); // Color.lightGray
		xyplot.setDomainGridlinePaint(Color.lightGray);
		xyplot.setRangeGridlinePaint(Color.lightGray);
		xyplot.setAxisOffset(new RectangleInsets(4D, 4D, 4D, 4D));
		xyplot.addRangeMarker(intervalmarker, Layer.BACKGROUND);
		// Chart (putting all together)
		JFreeChart jfreechart = new JFreeChart("NoLS predictions per residue",
				JFreeChart.DEFAULT_TITLE_FONT, xyplot, false);
		jfreechart.setBackgroundPaint(Color.white);
		File img = new File(imgPath, System.nanoTime() + ".png");

		ChartUtilities.saveChartAsPNG(img, jfreechart, imgSize.width,
				imgSize.height);
		return img.getName();
	}

	private static XYDataset convertData(double[] data) {
		XYSeries xyseries = new XYSeries("Series1");
		for (int i = 0; i < data.length; i++) {
			xyseries.add(i, data[i]);
		}
		DefaultXYDataset dataset = new DefaultXYDataset();
		dataset.addSeries(xyseries.getKey(), xyseries.toArray());
		return dataset;
	}

	private static double[] createSampleData() {
		return new double[] { 0.2, 0.4, 0.5, 0.9, 0.2, 0.2, 0.4, 0.5, 0.9, 0.2,
				0.2, 0.4, 0.5, 0.9, 0.2 };
	}

	public static void main(String args[]) throws IOException {
		NolsChart nolschart = new NolsChart(".");
		nolschart.createChart(createSampleData());
	}
}
